/*
 * Comparator.c
 *
 * Created: 17.04.2014 23:32:19
 *  Author: DL8NCI
 */ 


#include "Comparator.h"
#include "Keys.h"
#include "helpers.h"
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <stdio.h>
#include <avr/pgmspace.h>


#define CHARGE cbi(PORTD,5);
#define DISCHARGE sbi(PORTD,5);

// counter value and ready status for interrupt
volatile uint32_t CNT;
volatile uint16_t cnt1;
volatile uint8_t ready;
volatile uint32_t n_CNT;
double v0;		// Voltage across C in reset state
double v23;		// upper switch level - ca.2/3 v+
double f;		// ln((5V-v0)/(v23-5V))


// the timer1-capture interrupt
ISR(TIMER1_CAPT_vect) {
	uint16_t h;
	cli();
	cbi(TIMSK,TOIE1); // stop Overflow interrupt

	TCCR1B = 0;		// stop timer

	// read counter and add to global counter
	h = ICR1;
	cnt1 = h;
	CNT = CNT + h;
	n_CNT++;
	ready = 1;
	}


ISR(TIMER1_OVF_vect) {
	cli();
	cbi(TIMSK,TICIE1);		// stop capture interrupt

	TCCR1B = 0;		// stop timer
	ready = 2; // timeout
	}



//delay via Timer/Counter 1 but w/o interrupt
void delay(uint16_t d) {
	// normal mode is good enough
	TCCR1A = 0;
	TCCR1B = 0; // clock off
	TCNT1 = 0;
	TCCR1B = 1;
	while (TCNT1<d) idle(); // wait a couple of clocks
	}


int32_t ADC_conv(uint8_t n) {
	int32_t h;
	uint8_t i;
	int16_t c;
	
	h=0;
	for(i=0; i<n; i++) {
		sbi(ADCSRA,6);	// start conversion
		while ((ADCSRA & 0x40)!=0x00) idle();
		c = ADC;
		h = h + (int32_t)c;
		}
	return h;
	}

// ADC-conversion in polling mode
void ADC_conv_v0(uint8_t n) {
	int32_t h1,h2;
	
	cbi(SFIOR,3);	// MUX zurck zum ADC
	ADCSRA = 0x87;
	_delay_ms(1);
	
	ADMUX = 0xcd;	// 2.54V V-ref; (AD3-AD2)x10
	h1 = ADC_conv(n);
	
	ADMUX = 0xcc;	// 2.54V V-ref, (AD2-AD2)x10	
	h2 = ADC_conv(n);

	v0 = (double)(h1-h2)/((double)n*2000.0);

	printf_P(PSTR("%ld %ld %8.4f    "),h1,h2,v0);
	
	}




void Measure_555(void) {
	cli();						// disable all interrupts
	TCCR1A = 0x00;				// prepare timer 1 and ...
	TCCR1B = 0;					// ... stop timer 1

	TCNT1H = 0;					// set timer1 count value to 0
	TCNT1L = 0;

	// start measurement
	ready = 0;					// clear the ready status
	sbi(TIFR,ICF1);				// terminate active interrupts, if existing
	sbi(TIFR,TOV1);				// terminate active interrupts, if existing
	sbi(TIMSK,TICIE1);			// timer1-capture Interrupt on
	sbi(TIMSK,TOIE1);			// timer1-overflow interrupt on (timeout)

	sbi(SFIOR,PSR10);			// reset prescaler
	TCCR1B=0x01;				// start Timer 1
	
	ACSR = 0x47;
	CHARGE
	sei();						// enable interrupts
	_delay_us(2);

	while (ready==0) idle();	// wait until ready

	cbi(TIMSK,TICIE1);			// timer1-capture Interrupt off
	cbi(TIMSK,TOIE1);			// timer1-overflow interrupt 0ff

	DISCHARGE
	if (ready==1) {
		delay(cnt1 >> 2);		// wait ca. 1/4 of previous pulse length
		}
	
	poll();
	}




double Measure_555_N(void) {
	double t;

	CNT = 0;
	n_CNT = 0;
	ready = 0;
	
	DISCHARGE
	// 10 ms delay for stabilizing the relays
	_delay_ms(10.0);
	poll();

	
//	ADC_conv_v0(128);
	v0 = 0.0;
	v23 = 1.23;
	_delay_ms(1);

	// setup AC
	cbi(ADCSRA,7);		// ADC off
	sbi(SFIOR,3);		// MUX zum AC
	ADMUX = 0x03;
	
	// do as many measurements until 1e7 counts or more have been accumulated
	while (CNT < 3000000L) {
		Measure_555();
		if (ready==2) break;
		}
	
	cli();

	if (ready!=2) {
		t=(double)CNT/((double)F_CPU*(double)n_CNT);
		}
	else {
		t=-1.0;
		}
	return t;
}

	
//double NE555_get_v0(void) { return v0; }

double NE555_get_v23(void) { return v23; }
	
double NE555_get_lnf(void) {
	return log((5.0-v0)/(5.0-v23));
	}

void NE555_init(void) {
	// Trigger-Pin
	CHARGE
	sbi(DDRD,5);
	DISCHARGE
	
	// set up AC
	
	// set up ADC
	ADMUX = 0xc2;	// 2.54V V-ref; ADC2
	ADCSRA = 0x87;	//ADC on; prescaler = 128
	
	}
