/*
 * I2C.c
 *
 * Created: 11.05.2014 21:33:37
 *  Author: DL8NCI
 */ 

#include "I2C.h"
#include "TWI_Master.h"
#include <avr/interrupt.h>
#include "helpers.h"

static unsigned char twi_aux_1_data;
static unsigned char twi_aux_2_data;

void I2C_send_byte(uint8_t addr, uint8_t b) {
	static uint8_t messageBuf[4];
	messageBuf[0] = (addr<<TWI_ADR_BITS) | (FALSE<<TWI_READ_BIT);
	messageBuf[1] = b;
	sei();
	TWI_Start_Transceiver_With_Data( messageBuf, 2 );
    while ( TWI_Transceiver_Busy() );
	cbi(TWCR,TWIE);
	}

unsigned char I2C_receive_byte(unsigned char addr) {
	static unsigned char messageBuf[4];
	messageBuf[0] = (addr<<TWI_ADR_BITS) | (TRUE<<TWI_READ_BIT);
	sei();
	TWI_Start_Transceiver_With_Data( messageBuf, 2 );
	while (TWI_Transceiver_Busy()) { }
	TWI_Get_Data_From_Transceiver( messageBuf, 2 );
	cbi(TWCR,TWIE);
	return messageBuf[1];
	}

void I2C_cbi(uint8_t addr, uint8_t p) {
	switch (addr) {
		case TWI_AUX_1:
			twi_aux_1_data = twi_aux_1_data & ~(1<<p);
			I2C_send_byte(addr,twi_aux_1_data);
			break;
		case TWI_AUX_2:
			twi_aux_2_data = twi_aux_2_data & ~(1<<p);
			I2C_send_byte(addr,twi_aux_2_data);
			break;
		}
	}

void I2C_sbi(uint8_t addr, uint8_t p) {
	switch (addr) {
		case TWI_AUX_1:
			twi_aux_1_data = twi_aux_1_data | (1<<p);
			I2C_send_byte(addr,twi_aux_1_data);
			break;
		case TWI_AUX_2:
			twi_aux_2_data = twi_aux_2_data | (1<<p);
			I2C_send_byte(addr,twi_aux_2_data);
			break;
		}
	}

uint8_t I2C_tbi(uint8_t addr, uint8_t p) {
	return(I2C_receive_byte(addr) & (1<<p));
	}


void I2C_init() {
	TWI_Master_Initialise();
	twi_aux_1_data = 0xff;
	twi_aux_2_data = 0xff;
	I2C_send_byte(TWI_AUX_1,twi_aux_1_data);
	I2C_send_byte(TWI_AUX_2,twi_aux_2_data);
	}
	

