/*
 * LC_Meter_003.c
 *
 * Created: 09.03.2014 20:50:38
 *  Author: DL8NCI
 */ 

#include <stdio.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "USART.h"
#include "OLED.h"
#include "LM311.h"
#include "NE555.h"
//#include "Comparator.h"
#include "Keys.h"
#include "I2C.h"
#include "Hardware.h"


// measuring mode, calibration mode and range
uint8_t mmode;

double dc_1k, dc_33k, dc_1meg;	// C-Offsets fr die drei NE555-Mebereiche

// some fixed values
double C470 = 470e-12;
double C330 = 330e-12;
double C3 = 21.7e-12;



#define PI 3.1415926535
#define LN3 1.098612289




void printL(double L) {

	if      (L>=1e1 ) { printf_P(PSTR("%7.2f  H"),L);}
	else if (L>=1e0 ) { printf_P(PSTR("%7.3f  H"),L);}
	else if (L>=1e-1) { printf_P(PSTR("%7.1f mH"),L*1e3);}
	else if (L>=1e-2) { printf_P(PSTR("%7.2f mH"),L*1e3);}
	else if (L>=1e-3) { printf_P(PSTR("%7.3f mH"),L*1e3);}
	else if (L>=1e-4) { printf_P(PSTR("%7.1f uH"),L*1e6);}
	else if (L>=1e-5) { printf_P(PSTR("%7.2f uH"),L*1e6);}
	else if (L>=1e-6) { printf_P(PSTR("%7.3f uH"),L*1e6);}
	else              { printf_P(PSTR("%7.4f uH"),L*1e6);}
	}

void printL5(double L) {

	if      (L>=1e1 ) { printf_P(PSTR("%5.2f  H"),L);}
	else if (L>=1e0 ) { printf_P(PSTR("%5.3f  H"),L);}
	else if (L>=1e-1) { printf_P(PSTR("%5.1f mH"),L*1e3);}
	else if (L>=1e-2) { printf_P(PSTR("%5.2f mH"),L*1e3);}
	else if (L>=1e-3) { printf_P(PSTR("%5.3f mH"),L*1e3);}
	else if (L>=1e-4) { printf_P(PSTR("%5.1f uH"),L*1e6);}
	else if (L>=1e-5) { printf_P(PSTR("%5.2f uH"),L*1e6);}
	else if (L>=1e-6) { printf_P(PSTR("%5.3f uH"),L*1e6);}
	else              { printf_P(PSTR("%5.1f nH"),L*1e9);}
}

void printC5(double C) {

	if      (C>=1e1 )  { printf_P(PSTR("%5.2f uF"),C*1e6);}
	else if (C>=1e-6 ) { printf_P(PSTR("%5.3f uF"),C*1e6);}
	else if (C>=1e-7)  { printf_P(PSTR("%5.1f nF"),C*1e9);}
	else if (C>=1e-8)  { printf_P(PSTR("%5.2f nF"),C*1e9);}
	else if (C>=1e-9)  { printf_P(PSTR("%5.3f nF"),C*1e9);}
	else if (C>=1e-10) { printf_P(PSTR("%5.1f pF"),C*1e12);}
	else if (C>=1e-11) { printf_P(PSTR("%5.2f pF"),C*1e12);}
	else if (C>=0)	   { printf_P(PSTR("%5.3f pF"),C*1e12);}
	else if (C>-1e-10) { printf_P(PSTR("%5.1f pF"),C*1e12);}
	else			   { printf_P(PSTR("----- pF"));}
	}





void init_all(void) {

	cli();
	
	LCD_Init();
	LCD_SetStdOut();
	LCD_SendCmd(0x80);
	printf_P(PSTR("DL8NCI"));
	LCD_SendCmd(0xc0);
	printf_P(PSTR("04/2014"));


	USART_Init();
	USART_SetStdOut();
	printf_P(PSTR("DL8NCI 04/2014\r\n"));
	LCD_SetStdOut();

	I2C_init();

	// all relays off
	REL_LX_OFF
	REL_L3_OFF
	REL_330_OFF
	
	REL_1K_OFF
	REL_33K_OFF
	REL_1M_OFF
	
	REL_NE555_OFF
	REL_LM311_OFF
	
	// all relays as out
	
	NE555_init();
		
	Keys_init();

	USART_SetStdOut();
    printf_P(PSTR("LC-Meter Evaluation (c) 2014 by DL8NCI\r\n"));

	Calibrate();			// determine t_gate

	mmode = 0;				// the initial measuring mode
//	cmode = 0;				// calibrating mode
//	rmode = 0;				// range
		
	dc_1k = 0.0;
	dc_33k = 0.0;
	dc_1meg = 0.0;
		
		
	}

/*
void get_cmd(void) {
	uint8_t b;
	if ((UCSRA & 0x80) == 0) return;
	b = UDR;
	
	switch (b) {
		case '0':
			mmode = 0;
			break;
		}
	
	}
*/


int main(void) {
	uint8_t i;
	double f1,f2,f3,f4,Lx,Lx2,L3,Cy;
//	double f5,f6;
	double w1,w2,w3,w4, Cx,Cx2;
	double h1, h2, h3, h4, h5, num, denom;
	double t2,R,dC;
	
	init_all();
	
	REL_NE555_OFF
	REL_LM311_ON


// forever ...	
    while(1) {
		
		
		
//		get_cmd();
		
		
		
		switch (mmode) {
			case 0:	// L-mode
				REL_1K_OFF
				REL_33K_OFF
				REL_1M_OFF
				
				REL_NE555_OFF
				REL_LM311_ON

				REL_L3_OFF
				REL_LX_OFF
				REL_330_OFF

			    for(i=0; i<10; i++) { _delay_ms(10.0); }
			    f1=measure_m(4);	// 47 uH + Lx; 470 pF + Cy
//				f1=measure_by_T(1.0);
				w1=2*PI*f1;

				REL_LX_ON
			    for(i=0; i<10; i++) { _delay_ms(10.0); }
			    f2=measure_m(4);	// 47 uH; 470 pF + Cy
//				f2=measure_by_T(1.0);
				w2=2*PI*f2;

				REL_330_ON
				for(i=0; i<10; i++) { _delay_ms(10.0); }
				f3=measure_m(4);	// 47 uH; 470 pF + 330 pF + Cy
//				f3=measure_by_T(1.0);
				w3=2*PI*f3;
				
				REL_LX_OFF
				for(i=0; i<10; i++) { _delay_ms(10.0); }
				f4=measure_m(4);	// 47 uH + Lx; 470 pF + 330 pF + Cy
//				f4=measure_by_T(1.0);
				w4=2*PI*f4;
/*				
				REL_L3_ON
				for(i=0; i<10; i++) { _delay_ms(10.0); }
				f5=measure_m(4);	// Lx; 470 pF + 330 pF + Cx

				REL_330_OFF
				for(i=0; i<10; i++) { _delay_ms(10.0); }
				f6=measure_m(4);	// Lx; 470 pF + Cx

				// f1, f2 f3, f4, f5, f6
			    printf_P(PSTR("   f1=%8.3f kHz   f2=%8.3f kHz   f3=%8.3f kHz   f4=%8.3f kHz   f5=%8.3f kHz   f6=%8.3f kHz   TG=%8.3f ms\r\n"),f1*1e-3,f2*1e-3,f3*1e-3,f4*1e-3,f5*1e-3,f6*1e-3,t_gate*1e3);
*/
				// f1, f2 f3, f4
				printf_P(PSTR("   f1=%8.3f kHz   f2=%8.3f kHz   f3=%8.3f kHz   f4=%8.3f kHz   TG=%8.3f ms\r\n"),f1*1e-3,f2*1e-3,f3*1e-3,f4*1e-3,t_gate*1e3);



/*
			    printf_P(PSTR("123: Lx="));
			    if (f1>100) {
			
					Lx = -((f2-f1)*(f2+f1)*(f3-f2)*(f3+f2))/(4*PI*PI*f1*f1*f2*f2*f3*f3*C330);
					Cy = -(f3*f3*C470-f2*f2*C470+f3*f3*C330)/((f3-f2)*(f3+f2));
					L3 = -((f3-f2)*(f3+f2))/(4*PI*PI*f2*f2*f3*f3*C330);

				    printL(Lx);
				    printf_P(PSTR("   Cy=%5.1f pF   L3=%7.2f uH\r\n"),Cy*1e12,L3*1e6);
					}
				else {
				    printf_P(PSTR("---\r\n"));
					}



			    printf_P(PSTR("124: Lx="));
			    if (f1>100) {
				    
					Lx = -((f2-f1)*(f2+f1)*(f4-f1)*(f4+f1))/(4*PI*PI*f1*f1*f2*f2*f4*f4*C330);
					Cy = -(f4*f4*C470-f1*f1*C470+f4*f4*C330)/((f4-f1)*(f4+f1));
					L3 = -((f4-f1)*(f4+f1))/(4*PI*PI*f2*f2*f4*f4*C330);

				    printL(Lx);
				    printf_P(PSTR("   Cy=%5.1f pF   L3=%7.2f uH\r\n"),Cy*1e12,L3*1e6);
			    }
			    else {
				    printf_P(PSTR("---\r\n"));
			    }


			    printf_P(PSTR("134: Lx="));
			    if (f1>100) {
				    
				    Lx = ((f4-f1)*(f4+f1)*(f4-f3)*(f4+f3))/(4*PI*PI*f1*f1*f3*f3*f4*f4*C330);
					Cy = -(f4*f4*C470-f1*f1*C470+f4*f4*C330)/((f4-f1)*(f4+f1));
					L3 = -((f4-f1)*(f4+f1))/(4*PI*PI*f1*f1*f3*f3*C330);

				    printL(Lx);
				    printf_P(PSTR("   Cy=%5.1f pF   L3=%7.2f uH\r\n"),Cy*1e12,L3*1e6);
			    }
			    else {
				    printf_P(PSTR("---\r\n"));
			    }


			    printf_P(PSTR("234: Lx="));
			    if (f1>100) {
				    
				    Lx = ((f3-f2)*(f3+f2)*(f4-f3)*(f4+f3))/(4*PI*PI*f2*f2*f3*f3*f4*f4*C330);
					Cy = -(f3*f3*C470-f2*f2*C470+f3*f3*C330)/((f3-f2)*(f3+f2));
					L3 = -((f3-f2)*(f3+f2))/(4*PI*PI*f2*f2*f3*f3*C330);

				    printL(Lx);
				    printf_P(PSTR("   Cy=%5.1f pF   L3=%7.2f uH\r\n"),Cy*1e12,L3*1e6);
			    }
			    else {
				    printf_P(PSTR("---\r\n"));
			    }


			    printf_P(PSTR("56:  Lx="));
			    if (f5>100) {
				    
				    Lx = ((f6-f5)*(f6+f5))/(4*PI*PI*f5*f5*f6*f6*C330);
					Cy = -(f6*f6*C470-f5*f5*C470-f5*f5*C330)/((f6-f5)*(f6+f5));

				    printL(Lx);
				    printf_P(PSTR("   Cy=%5.1f pF\r\n"),Cy*1e12);
			    }
			    else {
				    printf_P(PSTR("---\r\n"));
			    }
				
				
*/				
				if (f1>100) {

//					Siehe auch Maxima: LC-Meter3.wxm

					L3 = (w2*w2-w3*w3)/(w2*w2*w3*w3*C330);

					Cy = -C3-C470-w3*w3*C330/(w3*w3-w2*w2);
					
					h1=(w1-w4)*(w1+w4)/(w4*w4);
					h2=(w2-w1)*(w2+w1)/(w1*w1);
					h3=(w3-w4)*(w3+w4)/(w2*w2);
					h4=(w3-w2)*(w3+w2)/(w3*w3);
					h5=(w4*w4-w3*w3+w2*w2-w1*w1)*C330;
					Lx=-h1*h2*h3*h4/h5;
					
				
					h1=w2*w2/((w2-w1)*(w2+w1));
					h2=w3*w3/((w3-w2)*(w3+w2));
					h3=(w2*w4-w1*w3)/((w4-w1)*(w4+w1));
					h4=(w2*w4+w1*w3)/((w4-w3)*(w4+w3));
					Cx=-h1*h2*h3*h4*C330;
					
					Cy = -C3-C470-w3*w3*C330/(w3*w3-w2*w2);
					
					
					w1=w1*1e-6;
					w2=w2*1e-6;
					w3=w3*1e-6;
					w4=w4*1e-6;
					
					num = w2*w2*w2*w2*w3*w3*w3*w3*(w1*w3-w2*w4)*(w2*w4+w1*w3)*C330*C330-
					      w2*w2*w3*w3*(w3-w2)*(w3+w2)*(w2*w2*w4*w4*w4*w4-w1*w1*w4*w4*w4*w4+w2*w2*w3*w3*w4*w4-w1*w1*w3*w3*w4*w4+w1*w1*w2*w2*w4*w4+w1*w1*w1*w1*w4*w4-w1*w1*w2*w2*w3*w3-w1*w1*w1*w1*w3*w3)*C3*C330-
					      (w3-w2)*(w3-w2)*(w3+w2)*(w3+w2)*(w2*w2*w3*w3*w4*w4*w4*w4-w1*w1*w3*w3*w4*w4*w4*w4+w1*w1*w1*w1*w2*w2*w4*w4-w1*w1*w1*w1*w2*w2*w3*w3)*C3*C3;
					
					denom = w2*w2*(w2-w1)*(w2+w1)*w3*w3*(w3-w2)*(w3+w2)*(w4-w1)*(w4+w1)*(w4-w3)*(w4+w3)*C330;
					
					// strker abhngig von C3; 
					Cx2 = num/denom;
					
					
					num = -w2*w2*(w2-w1)*(w2+w1)*w3*w3*(w3-w2)*(w3+w2)*(w4-w1)*(w4+w1)*(w4-w3)*(w4+w3)*C330;
					
					denom = w1*w1*w2*w2*w2*w2*w3*w3*w3*w3*w4*w4*(w4*w4-w3*w3+w2*w2-w1*w1)*C330*C330+
					        2.0*w1*w1*w2*w2*w3*w3*(w3-w2)*(w3+w2)*w4*w4*(w2*w4-w1*w3)*(w2*w4+w1*w3)*C3*C330+
							w1*w1*(w3-w2)*(w3-w2)*(w3+w2)*(w3+w2)*w4*w4*(w2*w2*w3*w3*w4*w4-w1*w1*w3*w3*w4*w4+w1*w1*w2*w2*w4*w4-w1*w1*w2*w2*w3*w3)*C3*C3;
					
					// relativ unabhngig von C3 
					Lx2 = 1e-12*num/denom;
					
					if (L3<0) {
						mmode = 2;
						break;
						}

				    printf_P(PSTR("     Lx="));
				    printL(Lx);
				    printf_P(PSTR("   Lx2="));
				    printL(Lx2);
				    printf_P(PSTR("   Cy=%5.1f pF   L3=%7.2f uH   Cx=%7.2f pF   Cx2=%7.2f pF\r\n"),Cy*1e12,L3*1e6,Cx*1e12,Cx2*1e12);
					
					LCD_SetStdOut();
					LCD_SendCmd(0x80);
				    printL5(Lx2);
					LCD_SendCmd(0xC0);
//					printC5(Cx2);
				    printf_P(PSTR("L Normal"));
					USART_SetStdOut();
					
				    }
			    else {
					mmode = 2;
				    }


				break;
			case 2: 
				REL_L3_OFF
				REL_LX_OFF
				REL_330_OFF
				
				REL_NE555_ON
				REL_LM311_OFF
				
				if (cmode==1) {		// Offsets sind zu bestimmen
					REL_1K_OFF
					REL_33K_OFF
					REL_1M_ON
					R=1.0e6;
					t2=Measure_555_N();
					dc_1meg = t2/(NE555_get_lnf()*R);
					
					REL_1M_OFF
					REL_33K_ON
					R=33.0e3;
					t2=Measure_555_N();
					dc_33k = t2/(NE555_get_lnf()*R);
					
					REL_33K_OFF
					REL_1K_ON
					R=1.0e3;
					t2=Measure_555_N();
					dc_1k = t2/(NE555_get_lnf()*R);
					
					cmode = 3;
					}
				else if (cmode==2) {
					dc_1k = 0.0;
					dc_33k = 0.0;
					dc_1meg = 0.0;
					cmode = 0;
					}


				REL_1K_OFF
				REL_33K_OFF
				REL_1M_ON
				R=1.0e6;
				dC = dc_1meg;
				t2=Measure_555_N();
				
				if (t2<0) {
					REL_1M_OFF
					REL_33K_ON
					R=33.0e3;
					dC = dc_33k;
				    t2=Measure_555_N();

					if (t2<0) {
						REL_33K_OFF
						REL_1K_ON
						R=1.0e3;
						dC = dc_1k;
					    t2=Measure_555_N();

						if (t2<0.0) {
							mmode = 0;
							break;
							}
						}
					}

				if (t2>0) {
					Cx = t2/(NE555_get_lnf()*R)-dC;
					printf_P(PSTR("Cx = %9.2f pF   t2 = %9.2f us   R = %5.1f kOhm\r\n"),Cx*1e12,t2*1e6,R*1e-3);
				
					LCD_SetStdOut();
					LCD_SendCmd(0x80);
					printC5(Cx);
					}
				else {
					LCD_SetStdOut();
					LCD_SendCmd(0x80);
					printf_P(PSTR("Overflow"));
					}
			
				LCD_SendCmd(0xc0);
//				printf_P(PSTR("%5.3f ms"),t2*1e3);
				if (cmode==3) {
					printf_P(PSTR("C   ZERO"));
					}
				else {
					printf_P(PSTR("C Normal"));
					}

//				printf_P(PSTR("%8.6f"),NE555_get_lnf());
				USART_SetStdOut();
				break;
			}
		}
	}