/*
 * LM311.c
 *
 * Created: 10.04.2014 21:06:24
 *  Author: DL8NCI
 */ 



#include "LM311.h"
#include "Keys.h"
#include "helpers.h"
#include "USART.h"
#include "OLED.h"
#include "Mega32Aux.h"
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

//#define TF 250		// ca. 17.361111 ms (57.6 Hz)
#define TF 127		// 112.5 Hz (8.8888... ms) bei TC0=6 ==> 18.75 Hz bzw. 53.3333... ms - Achtung: "n-1"






volatile uint32_t CNT;
volatile uint8_t ready;
volatile uint32_t n_CNT;
volatile uint8_t beta2;
volatile uint8_t mn2;


// the timer2-compare interrupt
ISR(TIMER2_COMP_vect) {
	uint16_t l,h;

	TCCR1B = 0;		// stop counter
	TCCR2 = 0;		// stop timer

	// read counter and ...
	l = TCNT1L;
	h = TCNT1H;
	// ... add to global counter
	CNT = CNT + l + (h << 8);
	ready=1;
	}

ISR(TIMER0_COMP_vect) {
	uint16_t l,h;

	TCCR1B = WGM1_B_normal_bm | CS1_no_clock_bm;						// stop counter
	TCCR2 = WGM2_normal_bm | COM2_non_PWM_toggle_bm | CS2_no_clock_bm;	// stop timer

	// read counter and ...
	l = TCNT1L;
	h = TCNT1H;
	// ... add to global counter
	CNT = CNT + l + (h << 8);
	ready=1;
	}


ISR(TIMER1_COMPA_vect) {
	// letzer Software-Zhlimpuls: auf OCR1B umschalten
	if (beta2==1) {
		cbi(TIMSK,OCIE1A);
		sbi(TIMSK,OCIE1B);
		}
	beta2--;
	}

/*
ISR(TIMER0_OVF_vect) {
	mn2++;
	}
*/
	
	
void measure_int_1(void) {
	// setup timer 1 as counter and timer 2 + timer 0 as timer
	cli();

	// 1. TC1 - setup, clear and wait
	TCCR1A = COM1A_normal_bm | COM1B_normal_bm | WGM1_A_normal_bm;
	TCCR1B = WGM1_B_normal_bm | CS1_no_clock_bm;
	TCNT1 = 0;

	// 2. TC2 - setup, clear and wait
	TCCR2 = WGM2_normal_bm | COM2_non_PWM_toggle_bm | CS2_no_clock_bm;
	OCR2 = TF;				// 57.6 Hz
	TCNT2 = 0;

	if ((PIND & 0x80)!=0) sbi(TCCR2,FOC2);	// if OC2=1: FOC to force a toggle

	// 3. TC0 - setup, clear and start
	TCCR0 = WGM0_normal_bm | COM0_normal_bm | CS0_T0_falling_bm;
	TCNT0 = 0x00;
	OCR0 = 0x02;			// entspricht Teilung durch 3 - (n-1)

	sbi(TIFR,OCF0);					// terminate active interrupts, if existing
	sbi(TIMSK,OCIE0);				// timer0-compare interrupt on

	// 4. Reset Timer Prescaler
	sbi(SFIOR,PSR2);


	// 5. start counter TC1
	TCCR1B = WGM1_B_normal_bm | CS1_div_64_bm;

	// 6. start timer TC2
	TCCR2 = WGM2_CTC_bm | COM2_non_PWM_toggle_bm | CS2_div_1024_bm;	//  0x1f;

	ready = 0;

	sei();

	while (ready==0) idle();

	n_CNT++;
	// ready for next block

	cbi(TIMSK,OCIE0);				// timer0-compare interrupt off

	}


// some preparation - mainly determine t_gate (done once only)
void Calibrate(void) {

	sbi(DDRD,7);	// OC2 als output
	t_gate = 0.0;
	CNT = 0;
	measure_int_1();

	printf_P(PSTR("Calibration: TF=%u   F_CPU=%f   CNT=%f\r\n"),TF,(double)F_CPU,(double)CNT);

	t_gate = 64*(double)CNT/(double)F_CPU;
}


// measure one time the current frequency
void measure_1(void) {

	// prepare timer 1 as counter and timer 2 as timer
	cli();

	TCCR1A = COM1A_normal_bm | COM1B_normal_bm | WGM1_A_normal_bm;
	TCCR1B = WGM1_B_normal_bm | CS1_no_clock_bm;
	TCNT1 = 0;

	TCCR2 = WGM2_normal_bm | COM2_non_PWM_toggle_bm | CS2_no_clock_bm;
	OCR2 = TF;
	TCNT2 = 0;

	if ((PIND & 0x80)!=0x00) sbi(TCCR2,FOC2);


	TCCR0 = WGM0_normal_bm | COM0_normal_bm | CS0_T0_falling_bm;
	TCNT0 = 0x00;
	OCR0 = 0x02;			// Teilerfaktor 3 (n-1)

	sbi(TIFR,OCF0);			// terminate active interrupts, if existing
	sbi(TIMSK,OCIE0);		// timer0-compare interrupt on

	sbi(SFIOR,PSR2);		// Reset Timer Prescaler

	//start counter
	TCCR1B = WGM1_B_normal_bm | CS1_T1_rising_bm;

	//start timer
	TCCR2 = WGM2_CTC_bm | COM2_non_PWM_toggle_bm | CS2_div_1024_bm; //    0x1f;

	ready = 0;

	sei();

	while (ready==0) idle();

	n_CNT++;
	cbi(TIMSK,OCIE0);				// timer0-compare interrupt off
	
	poll();
	
	}




// execute the frequency measurement n times
double measure_m(uint8_t n) {
	double f;
	CNT = 0;
	n_CNT = 0;
	while (n_CNT<n) measure_1();
	cli();
	f=(double)CNT/((double)n*t_gate);
	return f;
	}
	
double estimate_f(void) { return measure_m(1); }
	
double measure_by_T(double df) {
	double f,f2,md,kd;
	uint32_t m,n;
	uint16_t gamma0,gamma1,gamma2,alfa1,alfa2,k;
	uint8_t beta1;
	
	double stat_sx, stat_sxx, stat_sigma, stat_x;
	uint32_t stat_xmin, stat_xmax;
	uint16_t stat_n;
	
	stat_sx = 0.0;
	stat_sxx = 0.0;
	stat_xmin = 100000000UL;
	stat_xmax = 0UL;
	stat_n = 0;
	
	while(1) {
	
	f=estimate_f();
	
	md=(f*f)/(df*(double)F_CPU);
	m = (uint32_t)md + 1;
	
	kd = (double)F_CPU*(double)m/(131072.0*f) + 0.5;
	k = (uint16_t)kd;
	kd = (double)k + 0.5;
	
	md = f*kd*131072.0/(double)F_CPU;
	m = (uint32_t)md + 1;
	 
	
	
	
	
	n = (uint32_t)F_CPU/(uint32_t)f;
	
	alfa1 = 65000U;
	beta1 =  (uint32_t)m / (uint32_t)alfa1;
	gamma1 = (uint32_t)m % (uint32_t)alfa1;
	
	gamma0 = 1000;

	if (gamma1<gamma0) {
		alfa2 = alfa1 - (gamma0-gamma1)/(uint16_t)beta1;
		beta2 = (uint32_t)m / (uint32_t)alfa2;
		gamma2= (uint32_t)m % (uint32_t)alfa2;
		}
	else {
		alfa2 = alfa1;
		beta2 = beta1;
		gamma2 = gamma1;
		}
	

/*	
	if ((beta1>0) && (gamma1<gamma0)) {
		alfa2 = alfa1 - (gamma0-gamma1)/(uint16_t)beta1;
		beta2 = (uint32_t)m/(uint16_t)alfa2;
		gamma2= (uint32_t)m % alfa2;
		}
	else {
		alfa2 = alfa1;
		beta2 = beta1;
		gamma2 = gamma1;
		}
*/	
//	if (stat_n==0) printf_P(PSTR("f =%10.3f kHz  m=%10ld  n=%10ld  alfa2=%10u  beta2=%5d  gamma2=%10u  gamma0=%10u\r\n"),f*1e-3,m,n,alfa2,beta2,gamma2,gamma0);
	
	
	
	
	
	cli();

	ready = 0;
	mn2=0;
	
	// TC1
	TCCR1A = COM1A_normal_bm | COM1B_normal_bm | WGM1_A_CTC_bm;
	TCCR1B = WGM1_B_CTC_bm | CS1_no_clock_bm;
	OCR1A = alfa2-1;
	OCR1B = gamma2;//-1;
	TCNT1 = 0;
	
	if (beta2==0) {
		// TC1 gengt; softwarezhler wird nicht bentigt
		sbi(TIMSK,OCIE1B);
		cbi(TIMSK,OCIE1A);
		} 
	else {
		// zunchst den Softwarezhler benutzen
		sbi(TIMSK,OCIE1A);
		cbi(TIMSK,OCIE1B);
		}

	sbi(TIFR,OCF1A);
	sbi(TIFR,OCF1B);


	// TC2
	TCCR2 = WGM2_normal_bm | COM2_non_PWM_toggle_bm | CS2_no_clock_bm;
	OCR2 = 0xff;
	TCNT2 = 0;
	if ((PIND & 0x80)!=0) sbi(TCCR2,FOC2);
	
	// keine Interrupts
	cbi(TIMSK,OCIE2);
	cbi(TIMSK,TOIE2);
	
	sbi(TIFR,OCF2);
	sbi(TIFR,TOV2);


	
	// TC0
	TCCR0 = WGM0_normal_bm | COM0_normal_bm | CS0_T0_falling_bm;
	TCNT0 = 0;
	
	// nur overflow interrupt an;
	sbi(TIMSK,TOIE0);
	cbi(TIMSK,OCIE0);
	
	sbi(TIFR,TOV0);
	sbi(TIFR,OCF0);


	MCUCR = MCUCR | 0x0c;	// INT1 rising edge
	//sbi(GIFR,7);
	GIFR = 0x80;
	sei();
	//sbi(GICR,7);
	GICR = 0x80;
	
	
	while (ready==0) { }
	
	_delay_us(100);
	
	cli();
		
	uint8_t c2,c0;
	uint32_t c2a;
	
	c2 = TCNT2;
	c0 = TCNT0;
	
	if ((PIND & 0x80)!=0) c2a = 256UL; else c2a = 0UL;
	CNT = (uint32_t)c2 + c2a + ((uint32_t)c0 * 512UL) + (uint32_t)mn2 * 131072UL;
		
	f2 = (double)F_CPU*(double)(m+1)/(double)CNT;
	
//	printf_P(PSTR("  f2=%10.3f kHz  mn2=%5d  c0=%5d  c2=%5d  CNT=%10lu\r\n"),f2*1e-3,mn2,c0,c2,CNT);
	
	printf_P(PSTR("f2= %8.3f kHz  mn2=%3d  c0=%3d  c2a=%3lu  c2=%3d  CNT=%8lu   f=%8.3f kHz  m=%10ld  n=%10ld  alfa2=%10u  beta2=%5d  gamma2=%10u\r\n"),f2*1e-3,mn2,c0,c2a,c2,CNT,f*1e-3,m,n,alfa2,beta2,gamma2,gamma0);	

	LCD_SetStdOut();
	LCD_SendCmd(0x80);
	printf_P(PSTR("%8.3f"),f2*1e-3);
	LCD_SendCmd(0xc0);
	printf_P(PSTR("%8lu"),CNT);
	USART_SetStdOut();
	
	stat_sx = stat_sx + (double)CNT;
	stat_sxx = stat_sxx + square((double)CNT);
	if (CNT<stat_xmin) stat_xmin = CNT;
	if (CNT>stat_xmax) stat_xmax = CNT;
	stat_n++;
	
	
	if (stat_n==100U) {
		stat_x = stat_sx / (double)stat_n;
		stat_sigma = stat_sxx/(double)stat_n - square(stat_x);
		stat_sigma = sqrt(stat_sigma);

//		printf_P(PSTR("Mittelwert=%10.2f   StdDev=%10.4f   Min=%10lu   Max=%10lu   sx=%f   sxx=%f   n=%d\r\n"),stat_x,stat_sigma,stat_xmin,stat_xmax,stat_sx,stat_sxx,stat_n);

		stat_sx = 0.0;
		stat_sxx = 0.0;
		stat_n = 0;
		stat_xmin = 100000000UL;
		stat_xmax = 0UL;
		}
	
	
	}
	return f2;
	}



