/*
 * NE555.c
 *
 * Created: 10.04.2014 21:42:20
 *  Author: DL8NCI
 */ 


#include "NE555.h"
#include "Keys.h"
#include "helpers.h"
#include "Hardware.h"
//#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <stdio.h>
#include <avr/pgmspace.h>

// counter value and ready status for interrupt
volatile uint32_t CNT;
volatile uint16_t cnt1;
volatile uint8_t ready;
volatile uint32_t n_CNT;

double v0;		// Voltage across C in reset state
double v23;		// upper switch level - ca.2/3 v+
double f;		// ln((5V-v0)/(v23-5V))


// the timer1-capture interrupt
ISR(TIMER1_CAPT_vect) {
	uint16_t h;
	cli();
	cbi(TIMSK,TOIE1); // stop Overflow interrupt

	TCCR1B = 0;		// stop timer

	// read counter and add to global counter
	h = ICR1;
	cnt1 = h;
	CNT = CNT + h;
	n_CNT++;
	ready = 1;
	}


ISR(TIMER1_OVF_vect) {
	cli();
	cbi(TIMSK,TICIE1);		// stop capture interrupt

	TCCR1B = 0;		// stop timer
	ready = 2; // timeout
	}


//delay via Timer/Counter 1 but w/o interrupt
void delay(uint16_t d) {
	// normal mode is good enough
	TCCR1A = 0;
	TCCR1B = 0; // clock off
	TCNT1 = 0;
	TCCR1B = 1;
	while (TCNT1<d) idle(); // wait a couple of clocks
	}


int32_t ADC_conv(uint8_t n) {
	int32_t h;
	uint8_t i;
	int16_t c;
	
	h=0;
	for(i=0; i<n; i++) {
		sbi(ADCSRA,6);	// start conversion
		while ((ADCSRA & 0x40)!=0x00) idle();
		c = ADC;
		h = h + (int32_t)c;
		}
	return h;
	}

// ADC-conversion in polling mode
void ADC_conv_v0(uint8_t n) {
	int32_t h1,h2;
	
	ADMUX = 0xc9;	// 2.54V V-ref; (AD1-AD0)x10
	h1 = ADC_conv(n);
	
	ADMUX = 0xc8;	// 2.54V V-ref, (AD0-AD0)x10	
	h2 = ADC_conv(n);

	v0 = (double)(h1-h2)/((double)n*2000.0);

//	printf_P(PSTR("%ld %ld "),h1,h2);
	
	}


// ADC-conversion in polling mode
void ADC_conv_v23(uint8_t n) {
//	int32_t h1,h2;
	
	v23 = 10.0/3.0;
	
//	ADMUX = 0x5c;	// 5V V-ref; (AD4-AD2)x1
//	h1 = ADC_conv(n);

//	ADMUX = 0x5a;	// 5V V-ref; (AD2-AD2)x1
//	h2 = ADC_conv(n);
	
//	v23 = (double)(h1-h2)/((double)n*102.4);

//	printf_P(PSTR("%ld %ld  "),h1,h2);
	
	}



// measure one time the pulse length of the NE555
void Measure_555_1(void) {
	cli();						// disable all interrupts
	TCCR1A = 0x00;				// prepare timer 1 and ...
	TCCR1B = 0;					// ... stop timer 1

	TCNT1H = 0;					// set timer1 count value to 0
	TCNT1L = 0;

	// start measurement
	ready = 0;					// clear the ready status
	sbi(TIFR,ICF1);				// terminate active interrupts, if existing
	sbi(TIFR,TOV1);				// terminate active interrupts, if existing
	sbi(TIMSK,TICIE1);			// timer1-capture Interrupt on
	sbi(TIMSK,TOIE1);			// timer1-overflow interrupt on (timeout)

	sbi(SFIOR,PSR10);			// reset prescaler
	TCCR1B=0x01;				// start Timer 1

	NE555_TRIGGER_LOW			// start NE555
	sei();						// enable interrupts
	_delay_us(2);
	NE555_TRIGGER_HIGH			// return to high level

	while (ready==0) idle();	// wait until ready

	cbi(TIMSK,TICIE1);			// timer1-capture Interrupt off
	cbi(TIMSK,TOIE1);			// timer1-overflow interrupt off

	if (ready==1) {
		delay(cnt1 >> 2);		// wait ca. 1/4 of previous pulse length
		}
	
	poll();
	}




// execute the pulse length measurement until 1e7 counts have been accumulated
double Measure_555_N(void) {
	double t;

	CNT = 0;
	n_CNT = 0;
	ready = 0;
	
	// 10 ms delay for stabilizing the relays
	_delay_ms(10.0);
	poll();

	// Reset NE555
	NE555_RESET_LOW
	_delay_ms(10);
	ADC_conv_v0(128);
	NE555_RESET_HIGH
	_delay_ms(10);
	ADC_conv_v23(128);
	_delay_ms(1);

	// do as many measurements until 1e7 counts or more have been accumulated
	while (CNT < 3000000L) {
		Measure_555_1();
		if (ready==2) break;
		}
	
	cli();

	if (ready!=2) {
		t=(double)CNT/((double)F_CPU*(double)n_CNT);
		}
	else {
		t=-1.0;
		}
	return t;
	}




	
//double NE555_get_v0(void) { return v0; }

double NE555_get_v23(void) { return v23; }
	
double NE555_get_lnf(void) {
	return log((5.0-v0)/(5.0-v23));
	}

void NE555_init(void) {
	// Reset-Pin
	NE555_RESET_HIGH
	NE555_RESET_DIR_OUT
	
	// Trigger-Pin
	NE555_TRIGGER_HIGH
	NE555_TRIGGER_DIR_OUT
	
	// set up ADC
	ADMUX = 0xc2;	// 2.54V V-ref; ADC2
	ADCSRA = 0x87;	//ADC on; prescaler = 128
	
	}
