/*=============================================================================*
 * OLED.c - provides driver for LC/OLED-display                                *
 * Copyright (C) 2008  Matthias Jordan, DL8NCI                                 *
 * For more information see <http://www.dl8nci.de>                             *
 *=============================================================================*
 * This program is free software; you can redistribute it and/or modify it     *
 * under the terms of the GNU General Public License as published by the Free  *
 * Software Foundation; either version 3 of the License, or (at your option)   *
 * any later version.                                                          *
 *                                                                             *
 * This program is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       *
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    *
 * more details.                                                               *
 *                                                                             *
 * You should have received a copy of the GNU General Public License along     *
 * with this program; if not, see <http://www.gnu.org/licenses/>.              *
 *-----------------------------------------------------------------------------*
 * Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der *
 * GNU General Public License, wie von der Free Software Foundation            *
 * verffentlicht, weitergeben und/oder modifizieren, entweder gem Version 3 *
 * der Lizenz oder (nach Ihrer Option) jeder spteren Version.                 *
 *                                                                             *
 * Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen *
 * von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die         *
 * implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FR EINEN         *
 * BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.     *
 *                                                                             *
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit        *
 * diesem Programm erhalten haben. Falls nicht,                                *
 * siehe <http://www.gnu.org/licenses/>.                                       *
 *=============================================================================*/




#include "OLED.h"
#include "helpers.h"
#include "Hardware.h"

#include <stdio.h>
#include <util/delay.h>
#include <avr/io.h>


// global variables for LC-display
uint8_t LCD_pos;
uint8_t LCD_esc;




// all the LCD stuff - connected to the printf statement
void LCD_PulseE(void) {
	DISPLAY_E_HIGH
	_delay_us(2);		// 2 us
	DISPLAY_E_LOW
	}

void LCD_WaitBusy(void) {
	uint8_t h=0x08;

	DISPLAY_E_LOW
	DISPLAY_RS_LOW
	DISPLAY_RW_LOW
	
	DISPLAY_PORT_WRITE( 0x00 )
	
	DISPLAY_PORT_DIR_IN

	while(h!=0) {
		DISPLAY_RW_HIGH

		DISPLAY_E_HIGH
		_delay_us(1);					// 1 us
		h = DISPLAY_PORT_READ & 0x08;		// Bit 7 einlesen
		DISPLAY_E_LOW

		_delay_us(1);
		LCD_PulseE();
		_delay_us(1);
		
		DISPLAY_RW_LOW
		}
	
	DISPLAY_E_LOW
	DISPLAY_RS_LOW
	DISPLAY_RW_LOW
	DISPLAY_PORT_WRITE( 0x00 )
	DISPLAY_PORT_DIR_OUT
	}

void LCD_SendCmd(char c) {
	
	LCD_WaitBusy();
	
	DISPLAY_PORT_WRITE( ((c >> 4) & 0x0f) )
	LCD_PulseE();

	DISPLAY_PORT_WRITE( (c & 0x0f) )
	LCD_PulseE();

	DISPLAY_PORT_WRITE( 0x00 )
	}

void LCD_SendChar_int(char c) {

	LCD_WaitBusy();
	
	DISPLAY_RS_HIGH

	DISPLAY_PORT_WRITE( ((c >> 4) & 0x0f) )
	LCD_PulseE();

	DISPLAY_PORT_WRITE( (c & 0x0f) )
	LCD_PulseE();

	DISPLAY_RS_LOW
	
	DISPLAY_PORT_WRITE( 0x00 )	
	}

int LCD_SendChar(char c, FILE *stream) {
	if (LCD_esc==0) {
		if (c!='\001') {
			LCD_SendChar_int(c);
		}
		else {
			LCD_esc = 1;
		}
	}
	else {
		switch(c) {
			case '\001':
			LCD_SendCmd(0x80);
			LCD_pos = 0;
			LCD_esc = 0;
			break;
			case '\002':
			LCD_SendCmd(0xc0);
			LCD_pos = 0;
			LCD_esc = 0;
			break;
			case '\003':
			LCD_SendCmd(0x94);
			LCD_pos = 0;
			LCD_esc = 0;
			break;
			case '\004':
			LCD_SendCmd(0xd4);
			LCD_pos = 0;
			LCD_esc = 0;
			break;
			case '\005':
			while(LCD_pos<20) LCD_SendChar_int(0x20);
			LCD_esc = 0;
			break;
		}
	}
	return 0;
}

void LCD_SetStdOut(void) {
	static FILE mystdout = FDEV_SETUP_STREAM(LCD_SendChar,NULL,_FDEV_SETUP_WRITE);
	stdout = &mystdout;
}

void LCD_Init(void) {
	//	uint8_t i;

	DISPLAY_E_LOW
	DISPLAY_RW_LOW
	DISPLAY_RS_LOW
//	DISPLAY_LED_LOW
	
	DISPLAY_E_DIR_OUT
	DISPLAY_RW_DIR_OUT
	DISPLAY_RS_DIR_OUT
//	DISPLAY_LED_DIR_OUT

	DISPLAY_PORT_WRITE( 0x00 )
	DISPLAY_PORT_DIR_OUT

	DISPLAY_PORT_WRITE( 0x03 )	
	LCD_PulseE();
	_delay_us(50);

	DISPLAY_PORT_WRITE( 0x03 )
	LCD_PulseE();
	_delay_us(50);

	DISPLAY_PORT_WRITE( 0x03 )
	LCD_PulseE();
	_delay_us(50);

	DISPLAY_PORT_WRITE( 0x02 )
	LCD_PulseE();
	_delay_us(50);
	
	LCD_SendCmd(0x28);
	LCD_SendCmd(0x08);
	LCD_SendCmd(0x06);
	LCD_SendCmd(0x17);
	LCD_SendCmd(0x01);
	LCD_SendCmd(0x02);
	LCD_SendCmd(0x0c);

	LCD_pos = 0;
	LCD_esc = 0;
	}






